# Gil Getter

[![Build Status](https://drone.mashffxiv.com/api/badges/MashPotato/gilgetter/status.svg)](https://drone.mashffxiv.com/MashPotato/gilgetter)

Attempt to solve the best items to craft to make gil quickly, and the fastest way to craft them.

* Considers current costs of crafting materials on market board so you can buy the materials
* Considers volume of sales / hidden cost of taking up retainer slot
* Uses the item's sale velocity to determine optimimum quantity to craft
* Allows you to mark items you want to craft and builds "shopping list" of materials so you can buy them all at once
* Puts a checkmark next to items you already have posted on your retainers to avoid crafting duplicates

## Build Instructions

This software is not yet packaged and readily available to be distributed, however you can attempt to build it yourself on your own machine and run your own instance of Gil Getter.  

1. Install Go from https://golang.org
2. Clone this repository / download the code
3. Optional: You can use the data dump provided or make your own with your own custom filters.  
    * Use garlandtools-dump utility provided, see garlandtools-dump/README.md
    * Move the data dump file into the crafting-recipes directory
4. Build - see below

Just build it like any other Go program:

```bat
go build .
```

## Customize Data Dump Instructions

The data dump provided is for Goldsmithing recipes level 80 and lower on the world Excalibur.  If you want to change the data to match your own uses, you must use the provided garlandtools-dump utility in the garlandtools-dump folder.  See the README.MD file in this folder for usage details.

## Run Instructions 

Some runtime environment variables need to be defined, such as the world and your retainer names.  You can set any number of retainer names to match for as long as the variable suffix starts at 0 and counts up in a continuous range (see below).

```bat
set world=excalibur

set retainer_0=Superbe
set retainer_1=Monamoure
set retainer_2=Mondieu

.\gilgetter.exe
```

On first run, it will take some time to initialize and pull in all the item ID to name mappings, as well as the initial data sync with Universalis, after this is completed it should be accessible at http://localhost:8080

## Workflow

Once you've built the application and are able to run it, the general workflow is

1. Run the application, initial refresh of price data from Universalis - depending on number of recipes in the crafting-recipes directory this may take a few seconds to almost an hour.  
2. A list of the best items to craft will be displayed - but this uses non-realtime data from Universalis, it may be stale.  If you're fine with that, continute to step 3.  If you want to have high confidence the suggested items are based on realtime market, it's best to run down the list and click the copy item name button, paste into market board search (while running an uploader such as XIVLauncher).  GilGetter will refresh the price after 30 seconds.  If you run down the list of items, you can be certain the suggested items are currently the best items to craft right now.  This will also show what items you already have posted by putting a checkmark next to them.
3. Wait 30 seconds, a refresh prompt will be displayed.
4. Run down the list of items you want to craft or post by clicking the copy name button next to the item.  You can mark the item to craft by pressing the '0' number button on your keyboard.  You can tag items with numbers 1-9 by pressing the corresponding key on your keyboard as well (useful for marking retainer slots if you store items on your retainer).
5. When you've selected items you want to craft, click the checkout button.  You'll be taken to a page showing the items you want to craft, suggested quantity (based on sale velocity) and below that a list of crafting materials and quantities.  
6. Buy the crafting materials
7. Craft the items
8. Post items (I like to only have one of each item at a time and store rest on retainers just for posting later)
9. Collect

## FAQ

#### Why are there no graphs/charts?

Graphics are for people that like stories and make decisions.  The point of this utility is that the analysis has been done, and decisions have already been made.  A generalized strategy that works well is baked into the ranking function.  This is not a tool for performing analysis on an item by item basis.  

#### What is the strategy?

GilGetter's strategy is value-add between the current price of crafting materials and the estimated sale price of a crafted item. It also contains weights for sale velocity and historical price (vs current price).  The sale velocity helps cover the hidden cost of taking up a retainer sell slot (i.e. an item which could bring a lot of profit, but only 2 of them are sold per real life year).  All the factors are ran through the ranking function - sortItemSlice() in the main.go file and a list of suggested items are displayed for which you can craft and make money with.  

